# Copyright (c) 2010 gocept gmbh & co. kg
# See also LICENSE.txt

import ConfigParser
import datetime
import email
import gocept.support.configuration
import gocept.support.notification
import gocept.support.rt
import gocept.support.sla
import gocept.support.utils
import logging
import pytz
import re
import sys
import zope.sendmail

pin_re = re.compile('^.*\+(?P<pin>[0-9]{5})@.*$')


no_sla_found = u"""
Dear Customer,

we could not find a valid SLA for you. If you are sure to have one,
please check if you provided the correct PIN.

Regards,
gocept support"""


no_support_period = u"""
Dear Customer,

you have sent a notification without having an active support period.
Please check your contract for valid support times.

We have created ticket #%s. However our support staff was not notified and will deal
with your inquiry when your next support period begins.

Regards,
gocept support"""


def incoming_support_mail(conf, recipient, support_mail):
    try:
        pin = pin_re.match(recipient).group('pin')
        sla = gocept.support.sla.SLA.from_pin(conf, pin)
    except (AttributeError, KeyError):
        # no sla for that pin or no pin found in address
        # ask the requestor for a correct pin
        gocept.support.notification.notify_email(
            support_mail['From'],
            'Re: %s' % support_mail['Subject'],
            no_sla_found,
            conf,
            {'In-Reply-To': support_mail['Message-ID']})
        return
    now = gocept.support.utils.now(sla.tz)
    try:
        period = sla.period(now)
    except KeyError:
        period = None
    subject = support_mail.get('Subject')
    try:
        rt = gocept.support.rt.RT(conf.get('rt', 'user'),
                                  conf.get('rt', 'password'),
                                  conf.get('rt', 'url'))
        ticket = rt.new_ticket({'Queue': conf.get('rt', 'queue'),
                                'Subject': subject,
                                'Requestor': support_mail['From']})
    except:
        ticket = None
    ticket_id = ticket['id'] if ticket else 'n/a'
    if period is None:
        # sla, but no period, notify the requestor
        gocept.support.notification.notify_email(
            support_mail['From'],
            'Re: %s' % support_mail['Subject'],
            no_support_period % ticket_id,
            conf,
            {'In-Reply-To': support_mail['Message-ID']})

    if ticket is not None:
        # bounce the incoming mail to created support ticket
        del support_mail['Subject']
        support_mail['Subject'] = '[gocept #%s] Fw: %s' % (ticket_id, subject)
        gocept.support.notification.send_email('support-comment@gocept.com', support_mail, conf)

    if period is not None:
        ticket_due = now + datetime.timedelta(minutes=period.response_time)
        ticket.edit({'Starts': ticket_due.astimezone(pytz.utc).strftime(
            rt.DATE_FORMAT)})
        notifications = gocept.support.notification.Notifications(conf)
        notifications.notify('%smin %s:%s #%s %s' % (
            period.response_time,
            ticket_due.hour,
            ticket_due.minute,
            ticket_id,
            subject), '')


def incoming_support_mail_main():
    config = gocept.support.configuration.Configuration()
    config.optparser.add_option('-r', '--recipient',
        help='Recipient of the given Mail. Can contain the PIN '
             'of a SLA (recipient+<PIN>@host.domain)')
    config()
    support_mail = email.message_from_file(sys.stdin)
    incoming_support_mail(config.config, config.options.recipient, support_mail)
